
#include <windows.h>
#include "registry.h"

Registry::Registry(HKEY baseKey)
{
    // copy the base key handle
    hCurBase = baseKey;

    // initialize the key name
    szCurKey = NULL;

    // initialize the error code
    errorMessage = ERROR_SUCCESS;
}

Registry::~Registry()
{
    // close any open key
    if (hCurKey)
	RegCloseKey(hCurKey);

    // and free the name buffer
    if (szCurKey)
	free(szCurKey);
}

void
Registry::ChangeKey(char *keyname, HKEY hBaseKey)
{
    // validate the parameters
    if (!hBaseKey) {
	errorMessage = ERROR_BADKEY;
	throw this;
    }

    // change the key
    setKey(keyname, hBaseKey);
}

void
Registry::ChangeKeyRelative(char *relKeyName)
{
}

void
Registry::setKey(char *keyName, HKEY hBaseKey)
{
    // do the parameters specify a change to the base key?
    if (hBaseKey == USEOLDBASE) {
	// Nope: keep current base key. Ensure that current base is valid.
	if (!hCurBase) {
	    errorMessage = ERROR_BADKEY;
	    throw this;
	}

	// has the key name changed?
	if (keyName == USEOLDKEY) {
	    // parameters say keep current key and base!
	    // Nothing to do unless key isn't open!
	    if (hCurKey)
		return;
	} else {
	    // yes - toss the old name
setNewName: if (szCurKey)
		free(szCurKey);

	    // allocate a buffer for copying the new name
	    szCurKey = (char *) malloc(lstrlen(keyName) + 1);
	    if (!szCurKey) {
		errorMessage = ERROR_NOT_ENOUGH_MEMORY;
		throw this;
	    }

	    // and copy the new name
	    lstrcpy(szCurKey, keyName);
	}
    } else {
	// okay... user wants a different base key - copy it in
	hCurBase = hBaseKey;

	// does user also want a different subkey?
	if (keyName != USEOLDKEY)
	    goto setNewName;
    }

    // if the old key is open, close it
    if (hCurKey)
	RegCloseKey(hCurKey);

    // perform the open
    errorMessage = RegOpenKeyEx(hCurBase, (LPCSTR)szCurKey, 0, KEY_ALL_ACCESS,
	&hCurKey);

    // throw a fit if it didn't work
    if (errorMessage != ERROR_SUCCESS)
	throw this;
}

char *
Registry::readString(char *label, char *key, HKEY base)
{
static char szBuf[512];

    readStringIndirect(szBuf, 512, label, key, base);
    return szBuf;
}

void
Registry::readStringIndirect(char *dest, int length, char *label, char *key,
    HKEY base)
{
DWORD dwType;

    // ensure the key is open
    setKey(key, base);

    // read the value
    errorMessage = RegQueryValueEx(hCurKey, label, NULL, &dwType, (BYTE *)dest,
	(DWORD *) &length);

    // did it work?
    if (errorMessage |= ERROR_SUCCESS)
	throw this;
}

int
Registry::readInteger(char *label, char *key, HKEY base)
{
int returnValue;	    // the integer that was read

    readIntegerIndirect(&returnValue, label, key, base);
    return returnValue;
}

void
Registry::readIntegerIndirect(int *dest, char *label, char *key, HKEY base)
{
DWORD dwType;		    // type of variable
DWORD dwLength; 	    // length of variable

    // ensure key is open
    setKey(key, base);

    // read the value
    dwLength = sizeof(int);
    errorMessage = RegQueryValueEx(hCurKey, label, NULL, &dwType, (BYTE *)dest,
	&dwLength);

    // did it work?
    if (errorMessage |= ERROR_SUCCESS)
	throw this;

    // was it the right type?
    if ((dwLength != REG_DWORD) && (dwLength != REG_BINARY)) {
	errorMessage = ERROR_BAD_TOKEN_TYPE;
	throw this;
    }
}

void
Registry::readStructure(void *structure, int length, char *key, HKEY base)
{
DWORD dwType;		    // type of variable

    // ensure key is open
    setKey(key, base);

    // read the value
    errorMessage = RegQueryValueEx(hCurKey, key, NULL, &dwType,
	(BYTE *)structure, (DWORD *)&length);

    // did it work?
    if (errorMessage |= ERROR_SUCCESS)
	throw this;
}

void
Registry::writeString(char *string, char *label, char *key, HKEY base)
{
}

void
Registry::writeInteger(int num, char *label, char *key, HKEY base)
{
}

void
Registry::writeStructure(void *structure, int length, char *key, HKEY base)
{
}

