/*------------------------------------------------------------------*\
						File Resource Class
						A.K.A. BigFile
\*------------------------------------------------------------------*/

#ifndef _FILERESHEADER
#define _FILERESHEADER

#include <windows.h>
#include "RJMFileT.h"
#include "mem.h"

#define FRSRC_SEQ	0		// Sequential Access
#define FRSRC_RND	1		// Random Access

// Used for Seeking Operation
#define FRSRC_BEGIN		0	// Top of file
#define FRSRC_CURRENT	1	// Current file position
#define FRSRC_END		2	// From end of file

#pragma pack(push, pre_FRSRC_pack)
#pragma pack(1)

struct FRSRC_FILEHEADER
{
	unsigned		id;
	unsigned		count;
};

struct FRSRC_FILEENTRY
{
	unsigned		id;
	long			size;
	long			location;
	char			filename[16];
};

struct FRSRC_INFO
{
	FRSRC_FILEENTRY	*entry;
	long			position;
	unsigned short	fileopened,eof;
};

class FRSRC
{
private:
	
	HANDLE				handle;			// Resource file handle
	long				maxFiles;		// Max files opened at once

	long				count;			// Total number of file count
	FRSRC_INFO			*info;			// Array of opened files
	FRSRC_FILEENTRY		*directory;		// Array of directory entries

	long	GetAvailableHandle(void)
	{
		if (handle)
		{
			for (long i=0; i<maxFiles; i++)
			{
				if (info[i].fileopened==0)
					return i;
			}
		}
		return -1;
	}

	// Located file by it's ID
	long	__fastcall LocateFile(unsigned id)
	{
		for (long i=0; i<count; i++)
		{
			if (id==directory[i].id)
			{
				return i;
			}
		}
		return -1;
	}

	// Located file by it's Filename
	long	__fastcall LocateFile(const char *str)
	{
		for (long i=0; i<count; i++)
		{
			if (!stricmp(str,directory[i].filename))
			{
				return i;
			}
		}
		return -1;
	}

	BOOL	__fastcall IsFileHandleValid(long h)
	{
		if (info)
		{
			if (h >= 0 && h < count)
			{
				return TRUE;
			}
		}
		return FALSE;
	}

public:

  FRSRC() { info=NULL; handle=NULL; directory=NULL; maxFiles = 0; info = 0; }
  ~FRSRC() { Close(); }

	// 	*** Open/Close resource file (BIGFILE) ***
	BOOL	Open(const char *filename, int type, int max);
	void	Close(void);

	// *** Individual File Handling ***
	BOOL	CloseFile(long h);
	long	OpenFile(const char *filename);	// Open file by it's Filename
	long	OpenFile(long id);			// Open file by it's File ID
	BOOL	Eof(long h);				// EOF encountered
	long	Read(long h, LPVOID buffer, long size);
	long	Seek(long h, long position, DWORD from);

	int		__fastcall Calibrate(DWORD secs, long chunkSize);
	long	__fastcall OpenFileEntry(long entryid);
	FRSRC_FILEENTRY *GetFileInformation(long h);
};

inline long __fastcall FRSRC::OpenFileEntry(long entryid)
{
	long nexthandle = GetAvailableHandle();
	if (nexthandle==-1)
	{
		OutputDebugString("bf: no handle avail\n");
			return -1;
	}

	info[nexthandle].fileopened = TRUE;
	info[nexthandle].position = 0;
	info[nexthandle].entry = &directory[entryid];
	return nexthandle;
}

inline FRSRC_FILEENTRY *FRSRC::GetFileInformation(long h)
{
	if (!IsFileHandleValid(h))
		return NULL;

	return info[h].entry;
}

#pragma pack(pop, pre_FRSRC_pack)

#endif	// _FILERESHEADER
