// FunkyBar.cpp
#include "fwl5.h"
#include <commctrl.h>
#include "resource.h"
#include "funkybar.h"

#include <assert.h>

#define IDC_TOOLBAR 1666
#define IDC_SLIDER 1667
#define IDC_TEXTBOX 1668

TBBUTTON funkyBarBtns[1] = {0,ID_PAUSE,TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0};
WNDPROC savedBarProc;
WNDPROC savedSliderProc;

static const int stextTop = 6;
static const int stextHeight = 16;
static const int stextLeft = 160;
static const int stextWidthDelta = 2;

//----------------------------------------------------------------
FunkyBar::FunkyBar(BaseWin* parent, int volSteps_)
	:BaseWin(0,"",parent->GetInnerRect(),0)
   ,statusFont(0)
   ,volRange(volSteps_)
{
	FunkyBar::parent=parent;
}

//----------------------------------------------------------------
FunkyBar::~FunkyBar(void)
{
  if (statusFont)
  {
    DeleteObject(statusFont);
  }
}

//----------------------------------------------------------------
LRESULT CALLBACK FunkyBarProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_HSCROLL:
			SendMessage(GetParent(hWnd),message,wParam,lParam);
			break;
	}
	return (savedBarProc(hWnd,message,wParam,lParam));	
}

//----------------------------------------------------------------
LRESULT CALLBACK SliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_SETFOCUS:
			SetFocus(GetParent(GetParent(hWnd)));
			return 0;
	}
	return (savedSliderProc(hWnd,message,wParam,lParam));	
}

//----------------------------------------------------------------
BOOL FunkyBar::Create(void)
{
  // supposedly, when the FunkyBar window gets destroyed its
  // children get destroyed too...
  HWND h;
  h = CreateToolbarEx(parent->GetHWND(),
    WS_CHILD | /*WS_VISIBLE |*/ CCS_BOTTOM,
    IDC_TOOLBAR,
    2,  // 1 bitmap
    Application::FWGetAppInstance(),
    IDB_BITMAP4,
    funkyBarBtns,
    1,
    16,
    16,
    16,
    16,
    sizeof(TBBUTTON));
  assert(h);
  if (h)
  {
    SetHWND(h);
    SetOuterRect();

    // subclass toolbar to pass on WM_HSCROLL msgs
    savedBarProc = (WNDPROC)SetWindowLong(h,GWL_WNDPROC,(LONG)FunkyBarProc);

    hWndSlider = CreateWindowEx( 0L,
      TRACKBAR_CLASS,
      "",
      WS_CHILD | WS_VISIBLE,
      26,0,125,24,
      h,
      (HMENU)IDC_SLIDER,
      Application::FWGetAppInstance(),
      NULL );

    SendMessage(hWndSlider, TBM_SETRANGE, 
      (WPARAM) TRUE,                   // redraw flag 
      (LPARAM) MAKELONG(0, volRange));  // min. & max. positions 
    SendMessage(hWndSlider, TBM_SETPAGESIZE, 
      0, (LPARAM) 3);                  // new page size 

    SendMessage(hWndSlider, TBM_SETSEL, 
      (WPARAM) FALSE,                  // redraw flag 
      (LPARAM) MAKELONG(0,volRange)); 
    SendMessage(hWndSlider, TBM_SETPOS, 
      (WPARAM) TRUE,                   // redraw flag 
      (LPARAM) volRange/2); 

    CRect barRect = GetInnerRect();
    int dooWidth = barRect.GetWidth()-stextLeft-stextWidthDelta;

    NONCLIENTMETRICS met;
    met.cbSize = sizeof(met);
    SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, &met, 0);

    statusHeight = met.lfStatusFont.lfHeight;
    if (statusHeight < 0) statusHeight = -statusHeight;
    statusHeight += 5;

    statusFont = CreateFontIndirect(&met.lfStatusFont);

    hWndDoohickey = CreateWindowEx( 0L,
      "STATIC",
      "Testing",
      WS_CHILD | WS_VISIBLE | SS_SUNKEN | SS_CENTER,
      stextLeft,stextTop,dooWidth,statusHeight,
      h,
      (HMENU)IDC_SLIDER,
      Application::FWGetAppInstance(),
      NULL );

    // subclass slider so it can deal with focus
    savedSliderProc = (WNDPROC)SetWindowLong(hWndSlider,GWL_WNDPROC,(LONG)SliderProc);

    if (statusFont)
    {
      SendMessage(hWndDoohickey, WM_SETFONT, (WPARAM)statusFont, MAKELPARAM(TRUE, 0));
    }
    return TRUE;
  }
  return FALSE;
}

//----------------------------------------------------------------
void FunkyBar::AutoSize(void)
{
  SendMessage(GetHWND(),TB_AUTOSIZE,0,0);
  SetOuterRect();
  CRect myRect = GetInnerRect();
  int dooWidth = myRect.GetWidth()-stextLeft-stextWidthDelta;
  MoveWindow(hWndDoohickey,stextLeft,stextTop,dooWidth,statusHeight,FALSE);
  InvalidateRect(hWndDoohickey, 0, 0);
}

//----------------------------------------------------------------
void FunkyBar::Message(string newMessage)
{
	SetWindowText(hWndDoohickey,newMessage.c_str());
}

//----------------------------------------------------------------
void FunkyBar::ChangeButton(ButtonState x)
{
	HWND hwnd=GetHWND();
   SendMessage(hwnd,TB_CHANGEBITMAP,ID_PAUSE,x==FB_PAUSE?0:1);
}

//----------------------------------------------------------------
void FunkyBar::SetOuterRect()
{
   CRect screenRect;
   GetWindowRect(GetHWND(), &screenRect);
   ScreenToClient(GetParent(GetHWND()),(LPPOINT)&screenRect.left);
   ScreenToClient(GetParent(GetHWND()),(LPPOINT)&screenRect.right);
   outerRect = screenRect;
}

//----------------------------------------------------------------
void FunkyBar::EnableButton()
{
  HWND hwnd=GetHWND();
  SendMessage(hwnd,TB_ENABLEBUTTON,ID_PAUSE,(LPARAM) MAKELONG(TRUE, 0));
}

//----------------------------------------------------------------
void FunkyBar::DisableButton()
{
  HWND hwnd=GetHWND();
  SendMessage(hwnd,TB_ENABLEBUTTON,ID_PAUSE,(LPARAM) MAKELONG(FALSE, 0));
}

void FunkyBar::IndicateVolume(int indicate)
{
  SendMessage(hWndSlider, TBM_SETPOS, TRUE, indicate);
}
