/**********************************************************************\
	  GDI Save class
\**********************************************************************/

#include "GDISave.h"

BOOL GDISave::Restore(HDC destdc, RECT *rc)
{
	BOOL result;
	RECT defaultRect={ 0,0,0,0 };
	if (dc==NULL || image==NULL)
		return FALSE;

	if (rc==NULL)
	{
		GetClientRect( WindowFromDC(destdc), &defaultRect );
		rc = &defaultRect;
	}

	int destWidth	= rc->right - rc->left + 0;
	int destHeight	= rc->bottom - rc->top + 0;

	result = StretchBlt(destdc,0,0,destWidth,destHeight,
			dc,rc->left,rc->top,width,height,SRCCOPY);

	return result;
}

BOOL GDISave::Save(HDC sourcedc, RECT *rc)
{
	BOOL result;
	RECT defaultRect={ 0,0,0,0 };
	Close();
	dc = CreateCompatibleDC(sourcedc);
  	if (dc==NULL)
	{
		return FALSE;
	}

	SetStretchBltMode(dc,COLORONCOLOR);
	if (rc==NULL)
	{
		GetClientRect( WindowFromDC(sourcedc), &defaultRect );
		rc = &defaultRect;
	}
	
	width	= rc->right- rc->left + 0;
	height	= rc->bottom - rc->top + 0;
	image = CreateCompatibleBitmap(dc,width,height);
	if (image==NULL)
	{
		Close();
		return FALSE;
	}

	HGDIOBJ gdierror=SelectObject(dc,image);
	if (gdierror==(HGDIOBJ)0 || gdierror==(HGDIOBJ)GDI_ERROR)
	{
		Close();
		return FALSE;
	}

	GdiFlush();
	result = BitBlt(dc,0,0,width,height,sourcedc,
		rc->left,rc->top,SRCCOPY);
	GdiFlush();
	return result;
}

void GDISave::Close(void) 
{ 
	if (dc)
	{
		DeleteDC(dc);
		dc=NULL;
	}
	if (image)
	{
		DeleteObject(image);
		image=NULL;
	}
}
