/*---------------------------------------------------------------------------*\
								Render Class
\*---------------------------------------------------------------------------*/
#ifndef _RENDERER
#define _RENDERER

#include <windows.h>
#include <vfw.h>
#include "dsnd.h"
#include "sptypes.h"
#include "fader.h"

#define RENDERER_READY	0x0001
#define	DISPLAY_ON		0x0002

class Renderer
{
private:
	int					status;
	HDRAWDIB 			dibHandle;
	HDC 				dc;
	HWND				hwnd;
   	BITMAPINFOHEADER	bitmapInfo;
    RECT				fullRect;
	int 				destX,destY;
	int 				destWidth,destHeight;
    int 				sourceWidth,sourceHeight;

	FaderControl		fader;
	FADETYPE			fadetype;		// NOFADE,FADEIN,FADEOUT
	int					fadeinc;		// Fader step increment
	int					faderlevel;		// Current event
	DSoundBuffer		*soundfader;	// Sound buffer to get faded

public:

	BOOL 	Start(void);
	BOOL 	Stop(void);
	BOOL 	End(void);
	BOOL 	Begin(HDC hDC);

	Renderer();
	~Renderer() { Close(); }
	
	BOOL Open(HWND h)
	{
		hwnd = h;
		dc = GetDC(hwnd);
		if (dc)
		{
			dibHandle = DrawDibOpen();
			if (dibHandle)
				return TRUE;
		}
		Close();
		Start();
		return FALSE;
	}
	
	void 	Close(void);

	HDC		DC(void) { return dc; }
	HWND	Handle(void) { return hwnd; }

    void	__fastcall Setup(BITMAPINFOHEADER *info,LPRECT rc=NULL);
	BOOL	__fastcall FormatDifferent(BITMAPINFOHEADER *info);

	BOOL	Ready(void) { return status & RENDERER_READY; }
	void	On(void) { status |= DISPLAY_ON; }
	void	Off(void) { status &= ~DISPLAY_ON; }

    BOOL 	__fastcall SetDisplayAttr(LPRECT rect);
	BOOL 	__fastcall FastDraw(LPVOID data) { return Draw(data,DDF_HURRYUP); }
	BOOL	__fastcall Draw(LPVOID data,UINT flag=0);

	void	__fastcall Fade(FADETYPE t, int step, DSoundBuffer *snd);
};

#endif
