#ifndef __REGKEY_H
#define __REGKEY_H
// requires windows.h
// requires cstring.h or std string class.
class RegKey
{
public:
  RegKey(HKEY par, const string& sub);
  RegKey(HKEY par, const char* sub);
  RegKey(const RegKey& other);
  RegKey();
  RegKey& operator=(const RegKey& other);
  ~RegKey();
  int Opened() const;
  void Open();
  void Create();
  DWORD GetDWORD(const string& name);
  string GetSubKey(int index);
  string GetValue(const char* name);
  void GetValue(const char* name, char* valueDest,int valueSize);
  string GetValue(const string& value);
  void SetValue(const string& name, const string& value);
  void SetValue(const char* name, const char* value,int valueSize);
  void SetValue(const char* name, const char* value);
private:
  void Copy(const RegKey& other);
  HKEY myKey;
  const char* subKey;
  HKEY parent;
};
inline RegKey::RegKey()
{
  myKey=0;
  subKey =0;
  parent =0;
}
inline void RegKey::Copy(const RegKey& other)
{
  subKey = other.subKey;
  parent = other.parent;
  if (other.myKey)
  {
    long ec = RegOpenKeyEx( parent
                          , subKey
                          , 0
                          , KEY_ALL_ACCESS
                          , &myKey
                          );
    if (ec != ERROR_SUCCESS)
    {
      myKey = 0;
    }
  }
}
inline RegKey::RegKey(const RegKey& other)
{
  Copy(other);
}
inline RegKey& RegKey::operator=(const RegKey& other)
{
  if (myKey)
  {
    RegCloseKey(myKey);
  }
  Copy(other);
  return *this;
}
inline void RegKey::SetValue(const string& name, const string& value)
{
  SetValue(name.c_str(), value.c_str());
}
inline string RegKey::GetValue(const string& value)
{
  return GetValue(value.c_str());
}
inline RegKey::RegKey(HKEY par, const string& name)
  : parent(par)
  , subKey(name.c_str())
  , myKey(0)
{
}

#endif

