#ifndef __CPNET_H
#define __CPNET_H

class FRSRC;
class ifstream;
class ofstream;

class CPNetwork
{
public:
  CPNetwork();
  ~CPNetwork();
  int ReadFromFile(const string& fn);
  int ReadFromArchive(FRSRC* bigfile, int index);
  unsigned GetBaseAddress() const;
  int ReadFromRAM(BYTE* mem, int bytes);
  CPNode* GetFirstNode();
  CPNode* GetCompatibleLink(CPNode* iLink, const Psych& psych, int final=FALSE) const;
  const char* GetClipName(CPNode* node) const;
  CPNode* GetNextLeadout(CPNode* node, const Psych& psych) const;


  void SaveCounts(ofstream& out) const;
  void ReadCounts(ifstream& in);
  void ZeroCounts();
private:
  void WalkAllNodes(int (CPNetwork::*fn)(CPNode&,void*), void* user);
  void WalkAllNodes(int (CPNetwork::*fn)(CPNode&,void*), void* user) const;
  void Walk(CPNode* n,int (CPNetwork::*fn)(CPNode&,void*), void* user);
  void Walk(CPNode* n,int (CPNetwork::*fn)(CPNode&,void*), void* user) const;
  int SaveACount(CPNode& n, void* stream);
  int ZeroACount(CPNode& n,void*);
  CPNode* firstNode;
  struct StringTab
  {
    int n;
    char** strings;
    StringTab()
      : n(0)
    {
    }
    ~StringTab()
    {
      Clear();
    }
    void Clear()
    {
      for (int i=0; i<n; i++)
      {
        delete[] strings[i];
      }
      if (n)
      delete [] strings;
    }
  };
  StringTab clipNames;
  StringTab voxNames;
  //const char** clipNames;
  //const char** voxNames;
  BYTE* fileData;
  unsigned fileSize;
  BOOL UnFlattenVoidPtr(void*& ptr)
  {
    unsigned z = (BYTE*)ptr - fileData ;
    if (z > fileSize)
    {
      ptr = (void*)(fileData+(long)ptr);
      return FALSE;
    }
    return TRUE;
  }
  int Arc2Vector(void* fp, int handle, StringTab& tab);
  int File2Vector(void* fp, StringTab& tab);
#define DeclareUnFlattenPtrFunction(T) \
  BOOL UnFlattenPtr(T& ptr) \
  {                           \
    void* p = ptr;              \
    BOOL rv = UnFlattenVoidPtr(p); \
    ptr = (T)p;                    \
    return rv;                    \
  }                               
  BOOL UnFlattenPtr(char*& ptr)
  {
    void* p = ptr;
    BOOL rv = UnFlattenVoidPtr(p);
    ptr = (char*)p;
    return rv;
  }
  DeclareUnFlattenPtrFunction(CPNode*);
  DeclareUnFlattenPtrFunction(CPNode**);
  DeclareUnFlattenPtrFunction(CPTrack*);
  DeclareUnFlattenPtrFunction(CPSegment*);
  DeclareUnFlattenPtrFunction(Psych*);

  void UnFlatten(CPNode*);
};
inline unsigned CPNetwork::GetBaseAddress() const
{
  return unsigned(fileData);
}
inline const char* CPNetwork::GetClipName(CPNode* node) const
{
  if (node->data.film.clip > 65535) return ".";
  return clipNames.strings[node->data.film.clip];
}
#endif