#ifndef __DIBSECT_H
#define __DIBSECT_H
#include "cpoint.h"
#include "crect.h"
#include "csize.h"
#include "cdc.h"

// Windows color masks for 565 rgb.
#define RED565 0xf800
#define GRN565 0x07e0
#define BLU565 0x001f

#define RED555 0x7c00
#define GRN555 0x03e0
#define BLU555 0x001f

class DibSection
{
public:
  enum Type { T565, T555, T8 };
  class DibSectionRep
  {
  public:
    DibSectionRep(BYTE* source, const CSize& size, Type type);
    virtual ~DibSectionRep();
    BYTE* GetData() const { return data; }
    void Blit( const CDC& destDC
             , const CPoint& dest
             , const CRect& src
             ) const;
    void StretchBlit( const CDC& destDC
                    , const CRect& dest
                    , const CRect& src
                    ) const;
    unsigned GetBytesPerRow() const;
    HDC GetDC();
  private:
    unsigned bytesPerRow;
    friend class DibSection;
    BYTE* data;
    HDC bitmapDC;
    HBITMAP bitmap;
    int refs;
  };
  DibSection()
  {
    rep = 0;
  }
  virtual ~DibSection()
  {
    LoseOne();
  }
  DibSection(BYTE* source, const CSize& size, Type type)
  {
    rep = new DibSectionRep(source,size,type);
    rep->refs = 1;
  }
  DibSection(const DibSection& other)
  {
    rep = other.rep;
    GainOne();
  }
  DibSection& operator=(const DibSection& other)
  {
    if (rep != other.rep)
    {
      LoseOne();
      rep = other.rep;
      GainOne();
    }
    return *this;
  }
  DibSectionRep& operator*() { return *rep; }
  DibSectionRep* operator->() { return rep; }
  const DibSectionRep* operator->() const { return rep; }
private:
  void GainOne()
  {
    if (rep) rep->refs++;
  }
  void LoseOne()
  {
    if (rep)
    {
      rep->refs--;
      if (rep->refs==0)
      {
        delete rep;
      }
    }
  }
  DibSectionRep* rep;
};
inline unsigned DibSection::DibSectionRep::GetBytesPerRow() const
{
  return bytesPerRow;
}
inline HDC DibSection::DibSectionRep::GetDC()
{
  return bitmapDC;
}
#endif
