class EnabledMap
{
public:
  EnabledMap();
  virtual ~EnabledMap() {}
  void Enable(UINT id);  // enable an id
  void Disable(UINT id); // disable an id
  BOOL IsEnabled(UINT id); // nz if id is enabled.
  int GetDisabledCount();
  UINT GetDisabledItem(int k);
private:
  enum { kIds=64 };
  int nDisabled;
  UINT ids[kIds]; // ids of disabled commands
                  // policy is to say "enabled" if id not here.
};
inline int EnabledMap::GetDisabledCount()
{
  return nDisabled;
}
inline UINT EnabledMap::GetDisabledItem(int k)
{
  return ids[k];
}
inline BOOL EnabledMap::IsEnabled(UINT id)
{
  for (int i=0; i<kIds; i++)
  {
    if (ids[i]==0 || ids[i] > id)
    {
      return TRUE; // not here, so it's enabled.
    }
    if (ids[i]==id)
    {
      return FALSE; // it's here, so it's disabled.
    }
  }
  return TRUE;
}
inline EnabledMap::EnabledMap()
{
  memset(ids, 0, kIds*sizeof(UINT));
  nDisabled = 0;
}

inline void EnabledMap::Enable(UINT id)
{
  // make sure command is not present.
  for (int i=0; i<kIds; i++)
  {
    if (ids[i] > id)
      break; // not found
    if (ids[i] == id)
    {
      // move down the others
      memmove(ids+i, ids+i+1, kIds-i-1);
      nDisabled--;
      break;
    }
  }
}

inline void EnabledMap::Disable(UINT id)
{
  // make sure command is present
  int i;
  for (i=0; i<kIds; i++)
  {
    if (ids[i] == id)
      return; // it's here. go home
    if (ids[i] > id)
      break; // it's not here.
    if (ids[i] == 0)
      break; // zero means not filled, so not here.
  }
  if (ids[i] == 0)
  {
    ids[i] = id;
  }
  else
  {
    memmove(ids+i+1, ids+i, kIds-i-1);
    ids[i] = id;
  }
  nDisabled++;
}