#include "fwl5.h"
#include "fbutton.h"
#include "resource.h"

HBITMAP GetDLLBitmap(int id, RECT& dims);

///////////////////////////////////////////////////////////
//  FunkyButton
///////////////////////////////////////////////////////////
FunkyButton::FunkyButton
  (BaseWin* par
  ,const CPoint& where
  ,int buttonIdx
  ,int cmd
  ,int acc
  )
{
  id = cmd;
  bmps[0] = GetDLLBitmap(buttonIdx, area[0]);
  bmps[1] = GetDLLBitmap(buttonIdx+1, area[1]);
  bmps[2] = GetDLLBitmap(buttonIdx+2, area[2]);
  ttlArea = area[0];
  UnionRect(&ttlArea, &ttlArea, &area[1]);
  UnionRect(&ttlArea, &ttlArea, &area[2]);
  area[0].Move(CPoint(area[0].left - ttlArea.left, area[0].top-ttlArea.top));
  area[1].Move(CPoint(area[1].left - ttlArea.left, area[1].top-ttlArea.top));
  area[2].Move(CPoint(area[2].left - ttlArea.left, area[2].top-ttlArea.top));
  if (cmd == ID_HELLO_OPTIONS)
  {
    // SLIMEY!
    area[1].Move(CPoint(area[1].left, area[1].top+1));
  }

  static char name[] = "&A";
  name[1] = acc;
  HWND h = CreateWindow("button",name,
    WS_CHILD|WS_VISIBLE|BS_OWNERDRAW|WS_TABSTOP,
    ttlArea.left, ttlArea.top, 
    ttlArea.GetWidth(),
    ttlArea.GetHeight(),
    par->GetHWND(),(HMENU)cmd,
    Application::FWGetAppInstance(),0
    );
  if (h) //BM_SETIMAGE
  {
    SetHWND(h);
    SetWindowLong(GetHWND(), GWL_USERDATA, (LONG)this);
    assert(GetWindowLong(GetHWND(), GWL_USERDATA)==(LONG)this);
    // subclass button so it does our highlite proc
    savedProc = (WNDPROC)SetWindowLong(GetHWND(),GWL_WNDPROC,(LONG)FunkyButton::SubclassProc);
    snd.SetReader(new ResourceReader);
  }
}

//-----------------------------------------------------------------------------
LRESULT CALLBACK FunkyButton::SubclassProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  LRESULT retval;
  FunkyButton* it=(FunkyButton*)GetWindowLong(hWnd, GWL_USERDATA);
  switch(message)
  {
  case WM_SETFOCUS:
    it->MakeNoise("GOTFOCUS");
    break;
  case WM_MOUSEMOVE:
    if(GetCapture())
    {
      // mouse already been captured by us
      POINT xy;
      xy.x = LOWORD(lParam);
      xy.y = HIWORD(lParam);
      RECT clientArea;
      GetClientRect(hWnd,&clientArea);
      if(!PtInRect(&clientArea,xy))
      {
        // mouse left
        ReleaseCapture();
        ::SetFocus(GetParent(hWnd));
      }
    }
    else
    {
      // capture mouse so we'll know when it's left us:
      SetCapture(hWnd);
      ::SetFocus(hWnd);
    }
    break;
  case WM_KEYUP:
    //nVirtKey = (int) wParam;    // virtual-key code 
    //lKeyData = lParam;          // key data 
    if (int(wParam)==VK_SPACE || int(wParam)==13)
    {
      it->MakeNoiseWait("CLICKED");
      if (int(wParam)==13)
      {
        // a space gets this treatment automatically.
        it->PokeCommand();
      }
    }
    return it->savedProc(hWnd, message,wParam,lParam);
    
  case WM_LBUTTONDOWN:
    {
      retval = it->savedProc(hWnd, message,wParam,lParam);
      it->MakeNoiseWait("CLICKED");
      it->PokeCommand();
    }
    return retval;
  case WM_LBUTTONUP:
    return 0;
  case WM_DESTROY:
    it->SetHWND(0);
    break;
  case WM_ERASEBKGND:
    return 1; // trying to prevent unsightly white flashes
  case WM_NCDESTROY:
    {
      WNDPROC wp = it->savedProc;
      SetWindowLong(hWnd, GWL_WNDPROC, (LONG)it->savedProc);
      //delete it; // Not what you'd normally do.
      return wp(hWnd, message,wParam,lParam);
    }
  }
  return (it->savedProc(hWnd, message,wParam,lParam));
}
FunkyButton::~FunkyButton()
{
  while (!snd.ReallyDonePlaying())
    ;
  snd.Close();
}
//-----------------------------------------------------------------------------
void FunkyButton::DrawItem(CDC cdc,int state,const CRect& drawRect)
{
  HDC codc;

  codc = CreateCompatibleDC(cdc.Get()); 
  SaveDC(codc);

  SelectObject(codc, hbmpBk);
  BitBlt(cdc.Get()
         , 0, 0
         , ttlArea.GetWidth(), ttlArea.GetHeight()
         , codc
         , ttlArea.left, ttlArea.top
         , SRCCOPY
         );

  int index=0;
  if (state & ODS_SELECTED) index = 2; // if selected 
  else if(state & ODS_FOCUS) index = 1; // if focused

  SelectObject(codc, bmps[index]);

  // Destination 
  BitBlt( cdc.Get(),         // destination DC 
          area[index].left, 
          area[index].top,
          area[index].GetWidth(), //drawRect.GetWidth(),
          area[index].GetHeight(), //drawRect.GetHeight(),
          codc,    // source device context 
          0, 0,      // x and y upper left 
          SRCCOPY  // raster operation 
         );

  RestoreDC(codc, -1);
  DeleteDC(codc); 
}
void FunkyButton::SetBkgnd(const HBITMAP& bk)
{
  hbmpBk = bk;
}

void FunkyButton::MakeNoiseWait(const char* name)
{
  snd.Stop();
  snd.Open(name);
  snd.Play(1);
  while (!snd.ReallyDonePlaying())
    ;
}

void FunkyButton::MakeNoise(const char* name)
{
  snd.Stop();
  snd.Open(name);
  snd.Play(1);
}

void FunkyButton::PokeCommand()
{
  WPARAM msgParm1 = MAKELONG(id,BN_CLICKED);
  ::SendMessage(GetParent(GetHWND()),  WM_COMMAND, msgParm1, (LPARAM)GetHWND());
}
