#ifndef __NAVCONTR_H
#define __NAVCONTR_H
#include "cpnodes.h"
#include "fontbits.h"
#include "disks.h"
class GameWindow;
class Film;
class Smartfile;
class DSoundBuffer;
class Renderer;
struct CPNode;
class FRSRC;
class PromptManager;
class FileManager;
class Scheduler;
class VoiceOverManager;

enum NCError { NC_NOKEY=1, NC_REFUSED, NC_NOFILE, NC_SUCCESS };
//-----------------------------------------------------------------------------
class NavController
{
public:
	BOOL QueryClose();
  NavController(const char* command);
  ~NavController();
  void CDInserted(int newdisk);
  void ChangeCD(int newdisk);
  void CheckNewOrRestore();
  void CloseTest();
  void DoGoCommand();
  void DoOptionsCommand();
  void DoRestoreCommand();
  void DoReminderCommand();
  void DoRestartCommand();
  void DoRetryCommand();
  void DoSaveCommand();
  void DoStatusCommand();
  void DoStatusCommand(BaseWin* par);
  void DoUnpauseCommand();
  BOOL GameOver();
  const DiskID& GetCDInfo(int disk) const;
  const DiskID& GetHDInfo() const;
  const string& GetRegKey() const;
  int Idle();
  void InitTest();
  BOOL IsPaused();
  BOOL IsCDPresent(int id) const;
  void Key(int code, int flags);
  void MakeSchedulerBehave();
  void MouseDown(const CPoint& place, int flags);
  void OnAppExit();
  void OnPaint(const CDC& dc, const CRect& r);
  void OnSize(int x, int y);
  BOOL OnWindowCreate();
  void Pause();
  void PausePlayback();
  BOOL PromptForCD(int index);
  void ReminderEnds();
  void ResumePlayback();
  NCError SetRegKey(const string& newKey);
  void SetVolume(int vol);
  void SizeWindow();
  void TakeABranch();
  void TalkTo(GameWindow*w);
  void TimeManagement();
  void Unpause();
  void VolumeSofter();
  void VolumeLouder();
  BOOL WantsDialogKeys();
private:
  void FileManagement();
  void PromptManagement();
  int volume;
  BOOL startedFromIcon;
  BOOL schedulerOpen;
  FileManager* fileManager;
  PromptManager* promptManager;
  VoiceOverManager* voiceOverManager;

  GameWindow* window;
  static NavController* it;
  Scheduler* scheduler;
  FRSRC*		bigfileCD; // Added by Rudy
  FRSRC*     bigfileHD;

  friend class PlayState;
  friend class FinalStatusState;
  friend class InitialCreditState;
  friend class SplashState;
  friend class CDState;
  friend class FinalCreditState;
  FontBits font;
  const char* command;
  string gameKey;
  string nameOfSavedGame;
  Disks disks;
};
inline const DiskID& NavController::GetCDInfo(int disk) const
{
  return disks[disk];
}
inline const DiskID& NavController::GetHDInfo() const
{
  return disks.GetHD();
}
inline const string& NavController::GetRegKey() const
{
  return gameKey;
}
#endif