#include "fwl5.h"
#include "prompts.h"
#include "gamewin.h"
#include "cpnodes.h"
#include "film.h"
#include "options.h"

static CRect promptBox(0,192,320,240);
extern GameOptions globalOptions;
//-----------------------------------------------------------------------------
void PromptManager::TakeDownPrompt()
{
  CRect cr = window->GetPromptArea();

  CWindowDC(window).PatBlt(promptBox, BLACKNESS);
  promptUp =FALSE;
  font->SetColor(FONT_WHITE);
}
//-----------------------------------------------------------------------------
void PromptManager::DrawCenteredPrompt
  ( const char* text
  , DWORD color
  )
{
  CRect cr = window->GetPromptArea();
  int promptTop = cr.top;
  if (promptTop > 200)
  {
    font->SetZoom(2);
  }
  else
  {
    font->SetZoom(1);
  }
  CSize s = font->MeasureString(text);
  CPoint promptAlign;
  promptAlign.x = (cr.left + cr.right)/2 - s.cx/2;
  promptAlign.y = (promptTop + cr.bottom)/2 - s.cy/2;
  font->SetColor(color);
  font->SetCP(promptAlign);
  font->DrawString(CWindowDC(window), text);
  promptBox.Move(promptAlign);
  promptBox.SetSize(s);
}
//-----------------------------------------------------------------------------
void PromptManager::Manage()
{
  if (promptUp && !promptColored)
  {
    if (mouse)
    {
      DrawCenteredPrompt(promptSeg.prompt, FONT_RED);
      promptColored = TRUE;
      redTime = GetTickCount();
    }
  }
  if (promptUp && promptColored)
  {
    if (GetTickCount()-redTime > 1000)
    {
      TakeDownPrompt();
    }
  }
  // Policy: Difficulty change won't take down prompt
  // immediately, but option change will.
  enabled = globalOptions.SubtitlesOn();
  if (promptUp && !enabled && lEnabled)
  {
    TakeDownPrompt();
  }
  lEnabled = enabled;
}
#include "clcontex.h"
//-----------------------------------------------------------------------------
void PromptManager::NotifyOn(const ClipContext& context)
{
  BOOL loop = context.loop;
  BOOL leadin = context.node->data.film.type == CP_LEADIN;
  CPSegment* seg = &context.node->data.film.pTracks[context.trackIndex].pSegments[context.segIndex];

  BOOL difEnabled = context.passCount > 0 || !loop;
  BOOL branchEnabled = !(loop && (disabledBranches & (1<<context.segIndex)));
  if (enabled && difEnabled)
  {
    promptSeg = *seg;
    if (promptUp && leadin)
    {
      TakeDownPrompt(); // we're gonna try leaving up the prompt longer.
    }
    if (promptSeg.prompt)
    {
      if (loop)
      {
        DrawCenteredPrompt(promptSeg.prompt, branchEnabled?FONT_GREEN:FONT_GRAY);
      }
      else
      {
        DrawCenteredPrompt(promptSeg.prompt, FONT_WHITE);
      }
      promptUp = TRUE;
    }
    promptColored = FALSE;
    mouse = 0;
  }
}

//-----------------------------------------------------------------------------
void PromptManager::NotifyOff()
{
 // another way to know it's time to erase a prompt
  // if the prompt was colored, then it should come down at NotifyOn or something.
  if (enabled && !promptColored)
  {
    TakeDownPrompt();
  }
}

void PromptManager::EnableAllBranches()
{
  disabledBranches = 0;
}

void PromptManager::DisableBranch(int kBranch)
{
  disabledBranches |= (1<<kBranch);
}

void PromptManager::StartHyper(char* prompt)
{
  DrawCenteredPrompt(prompt, FONT_WHITE);
}

void PromptManager::EndHyper()
{
  TakeDownPrompt();
}
