#ifndef __PSTATE_H
#define __PSTATE_H

class ofstream;
class ifstream;
#include "cpnodes.h"

class PlayerState
{
public:
  PlayerState()
    : nodesUsed(0)
    , nodesAllocated(0)
    , nodes(0)
  {
  }
  ~PlayerState()
  {
    if (nodes) free(nodes);
  }
  void VisitNode(CPNode* node, const Psych& psych, long score);
  CPNode* GetNodeAndRetry(Psych& newPsych, long& newScore);
  CPNode* GetRetryNode() const;
  CPNode* GetLastNode() const;
  void ForgetAllNodes();
  BOOL SaveNodes(ofstream& file, unsigned baseAddress) const;
  BOOL LoadNodes(ifstream& file, unsigned baseAddress);
private:
  unsigned GetRetryNodeIndex() const;
  friend class PlayerStateIterator;

  class PlayerStateElement
  {
  public:
    CPNode* node;
    Psych psych;
    long score;
  };
  void AllocateMoreNodes();

  PlayerStateElement* nodes;
  unsigned nodesAllocated;
  unsigned nodesUsed;
};

class PlayerStateIterator
{
public:
  PlayerStateIterator() : st(0),index(0) {}
  PlayerStateIterator(const PlayerState& st_) : st(&st_),index(0) {}
  PlayerStateIterator(const PlayerStateIterator& it_) 
    : st(it_.st)
    , index(it_.index) 
  {
  }
  PlayerStateIterator& operator=(const PlayerStateIterator& it)
  {
    st = it.st;
    index = it.index;
    return *this;
  }
  void Reset() { index=0; }
  void SetToEnd() { index = st->nodesUsed-1; }
  void Advance() { index++; }
  void Backup() { index--; }
  BOOL RDone() const { return Done(); } // unsigned!
  BOOL Done() const { return index >= st->nodesUsed; }
  CPNode* GetNode() { return st->nodes[index].node; }
  long GetScore() const { return st->nodes[index].score; }
  Psych GetPsych() const { return st->nodes[index].psych; }
private:
  const PlayerState* st;
  unsigned index;
};

#endif