/********************************************************************\

	All buffer sizes are in K-bytes.  16 is translated to 16 x 1024

\********************************************************************/
#ifndef _SCHEDULER
#define _SCHEDULER

#include <windows.h>
#include "main.h"

#define MAXNAMESIZE			255
#define	NODES				4

typedef enum { BYNAME,BYID,BYENTRY,NONESELECTED } CEVENTIDBY;

class CEvent
{
private:
//  int 		linkhandle;				/* Event Handle */
  int			loopCount;				/* Maximum number of loops */
  FADETYPE	effecttype;				/* Event fade/effect type */

  Film	*clip;					/* Film Clip */
  TLINK<EventStr>	event;				/* Events hooked */

  CEVENTIDBY	idtype;
  long		idname;					/* File ID or Entry ID */
  char 		fname[MAXNAMESIZE];		/* Filename */
  FILMTYPE  	type;					/* Film type */

public:
  ~CEvent() 
  { 
    if (clip) 
      clip->Close(); 
  }
  CEvent()
    : loopCount(0)
    , effecttype(NOFADE)
    , clip(0)
    , idtype(NONESELECTED)
    , idname(0)
    , type(SHOT)
  {
    fname[0]=0;
  }

  CEvent(char *name, FILMTYPE tp, int loops, FADETYPE effect)
    : loopCount(0)
    , effecttype(NOFADE)
    , clip(0)
    , idtype(NONESELECTED)
    , idname(0)
    , type(SHOT)
  {
    Setup(name,tp,loops,effect);
  }

  void LinkClip(Film& f);

  FILMTYPE GetFILMTYPE() const;

  // Assign FileID/EntryID overriding filename specifier
  void AssignID(long id, CEVENTIDBY type) { idtype=type; idname=id; }

  void Setup(char *name, FILMTYPE tp, int loops, FADETYPE effect)
  {
    idtype=BYNAME;
    if (name) // Only do this if name is assigned
      strcpy(fname,name);
    type=tp;
    loopCount=loops;
    effecttype=effect;
  }

  int __fastcall Open(SmartGroup *smarthandle, int loopevent)
  {
    switch(idtype)
    {
    case BYENTRY:
      return smarthandle->OpenEntry(idname,loopevent);
    case BYID:
      return smarthandle->Open(idname,loopevent);
    case BYNAME:
      return smarthandle->Open(fname,loopevent);
    default:
      return -1;
    }
  }

  int  AddTrigger(DWORD frame, FilmCallback fn, LPVOID param);
  BOOL RemoveTrigger(int h);

  BOOL Play(void) { return clip->Play(loopCount,effecttype); }
  void PlayToEnd(void) { clip->PlayToEnd(); }
  void Stop(void) { clip->Stop(); }
  void Close(void) { clip->Close(); }

  Film*		Clip(void) { return clip; }
  FILMTYPE	Type(void) { return type; }
  char*		Filename(void) { return fname; }

//  friend class Scheduler;
};

typedef CEvent	*PCEvent;

class Scheduler
{
private:
  FRSRC			*bigfile;

	BOOL			statusPlaying;
  BOOL			statusLOOPING;
  BOOL			statusOPEN;
  BOOL			statusPAUSED;

  BOOL			playnext;
  int				playfocus,loadfocus;
  int				leadin_count;
  int				leadinID[NODES];

  CEvent			*currentEvent;	/* Current event */
  PCEvent			*event;			/* Array of event pointers in memory */
  TLINK<CEvent>	eventlist;		/* Events to play */

  int				masterLoopCount;/* Maximum loops for LOOPs */

  ORDER_NO		user_sel;
  ORDER_NO		defaultLeadin;	/* Leading selected if loop timeout */

  PeriodicTimer	outputTimer;	// Video Sync TMR

  Renderer		renderer;
  DSoundBuffer	sound;
  SmartGroup		shandle;
  RAVOutput		ravout;
  Film		*clip;

	void		__fastcall CloseFocus(int &focus);
  void		WipeOtherLeadins(void);
	void		PurgeLeadinsFromList(void);
  CEvent *	GetNextEvent(void);
  CEvent *	GetEventByHandle(int which);
  BOOL		PlayNextEvent(void);
  BOOL		LoadNextEvent(void);
  BOOL		RemoveByHandle(int h);
  FILMTYPE	PeekAtNextEvent(void);
  void		Check(void);
  void ClearLeadinIDs();

public:
	void OpenArchive();
	void CloseArchive();

  Scheduler();
  ~Scheduler();

  BOOL 	Open(HWND, FRSRC *);
  void 	Close(void);
  void	SetupLoop(int loop=LOOP_FOREVER, ORDER_NO leadin=THIRD);

  void	Empty(void);
  CEvent *AddEntry(long entryid,FILMTYPE tp, FADETYPE effect=NOFADE);
  BOOL	Remove(CEvent *evt);

  int GetTotalEventsInCue(void)
  {
    return eventlist.count();
  }

  void	__fastcall SelectLeadin(ORDER_NO sel)
  {
    user_sel = sel;
  }

  int		LoopingStatus(void) 
  { 
    return leadin_count; 
  }

  CEvent	*Playing(void) { return currentEvent; }

	void Pause(void)
	{
		statusPAUSED = TRUE;
		ravout.Pause();
	}

	void Resume(void)
	{
		statusPAUSED = FALSE;
		ravout.Resume();
	}

  void	Manager(void)
  {
    if (statusOPEN)
    {
      ravout.UpdateAV();
      outputTimer.TimeElapsed();

      if (currentEvent)
        currentEvent->Clip()->Check();
        
      Check();
    }
  }

  // Assigns a bigfile to this object
  void __fastcall SetArchieveAttrib(FRSRC *rc)
  {
    bigfile=rc;
    shandle.SetArchieveAttrib(bigfile);
  }

  void ForgetAll(void);

  // This doesn't determine if it is paused
	BOOL IsClipPlaying() { return statusPlaying; }

  friend void FileChecker(LPVOID param);
  friend void OutputChecker(LPVOID param);
  friend void TriggerPlayNext(Film *film, LPVOID param, ReasonForCB reason);


  void DebugReport() { return; }

	void	__fastcall OnSize(int x, int y)
	{
		RECT rc = { 0,0,x-1,y-1 };
		renderer.SetDisplayAttr(&rc);
	}

	void Scheduler::OnPaint()
	{
		ravout.RefreshVideo();
	}
};

#endif
