/********************************************************************\

  Methods:

			BOOL		Setup(int buffers, int smallsize, int bigsize)
						Shutdown()

			int			Open(char *filename, BOOL loop)	// Return ID
  			BOOL		Close(int id);

			SmartFile * Data(int id)					// SmartFile Pointer

			BOOL		Check()

\********************************************************************/

#ifndef _SMARTHANDLEHEAD
#define _SMARTHANDLEHEAD

#include <assert.h>
#include <windows.h>
#include "FTimers.h"
#include "Sbuffer.h"
#include "FReader.h"

#define LOOP_BUFFER		0
#define OTHER_BUFFERS	1

// Smart is combining the buffer and reader so there is a better grip!
class SmartFile : public FileReader, public SmartBuffer
{
private:
		char			SyncName[32];
		HANDLE			syncHandle; // For synchonization
		HANDLE			threadSyncHandle;

		DWORD			threadId;
		HANDLE			threadHandle;
		BOOL			terminateThread;
		PeriodicTimer	checkTimer;

		BOOL __fastcall StreamDataFromFile(long start, PDWORD bytesRead);
		BOOL __fastcall Check(long amount=0);
public:
		SmartFile();
		~SmartFile();

		void __fastcall SpotCheck(int amount=0);
		void __fastcall SetFileReadPeriod(int per);

		friend static void SmartFileCheck(LPVOID param);
		friend static void AsynchFileHandler(LPVOID param);
};

class SmartGroup
{
private:
		FRSRC			*bigfile;
		SmartFile		*databuffer;			// FileReader/Buffer System in one neat class

		int				numOfBuffers;
		BOOL 			readerBusy;
		int				smallBuffer,bigBuffer;

		int		__fastcall GetFreeBuffer(BOOL big);

public:

		BOOL	Setup(int buffers, int smallsize, int bigsize);
		void	Shutdown(void);

		int		__fastcall Open(const char *filename, BOOL loop);
		int		__fastcall Open(long id, BOOL loop);
		int		__fastcall OpenEntry(long entryid, BOOL loop);

		void	__fastcall Close(int id)
		{
			assert(id >= 0 && id <= numOfBuffers);
			databuffer[id].CloseFile();
			databuffer[id].ResetBuffer();
		}
		
		SmartFile * __fastcall Data(int id) { return &databuffer[id]; }

		// Assigns a bigfile to this object
		void __fastcall SetArchieveAttrib(FRSRC *rc)
		{
			bigfile=rc;
			for (int i=0; i<numOfBuffers; i++)
				databuffer[i].SetArchieveAttrib(bigfile);
		}

		void __fastcall SetFileReadPeriod(int per);
};

#endif
