#ifndef _VOLUMES_H
#define _VOLUMES_H
struct VolumeID
{
  string volumeName;
  string friendlyName;
  string path;
};

class Volumes
{
public:
  Volumes()
    : nVolumes(0)
    , vols(0)
  {
  }
  ~Volumes()
  {
    if (vols) delete [] vols;
  }
  void Setup(const string& keyName);
  const VolumeID& operator[](unsigned v) const { assert(v<nVolumes); return vols[v]; }
  const VolumeID& GetHD() const { return hd; }
private:
  unsigned nVolumes;
  VolumeID* vols;
  VolumeID hd;
};

#include "regkey.h"
inline void Volumes::Setup(const string& keyName)
{
  RegKey dir(HKEY_CURRENT_USER, keyName);
  dir.Open();
  nVolumes = (unsigned)dir.GetDWORD("MooCount");
  unsigned i;

  if (nVolumes==0)
  {
    // special case for old layout:
    // No MooCount key, no labels, no vols.
    // Just one cmoo.
    nVolumes=1;
    vols = new VolumeID[1];
    vols[i].path = dir.GetValue("cmoo");
    vols[i].volumeName = "";
    vols[i].friendlyName = "The Disk";
    hd.path = dir.GetValue("hmoo");
  }
  else
  {
    vols = new VolumeID[nVolumes];

    for (i=0; i<nVolumes; i++)
    {
      char buf[20];
      sprintf(buf, "cmoo%d", i);
      vols[i].volumeName = dir.GetValue(buf);
      sprintf(buf, "cvol%d", i);
      vols[i].path = dir.GetValue(buf);
      sprintf(buf, "label%d", i);
      vols[i].friendlyName = dir.GetValue(buf);
    }
    hd.path = dir.GetValue("hmoo");
  }
};
#endif